-- SupertypesAndSubtypes_Objects_Script
use	Super
;
go

-- Common (Schema)
if	(schema_id('Common') is null)
	begin
		exec	(N'
		create schema	Common
			authorization	dbo
		;
			')
		;
	end
go

-- Person (Schema)
if	(schema_id('Person') is null)
	begin
		exec	(N'
		create schema	Person
			authorization	dbo
		;
			')
		;
	end
go


-- Person.PersonSparse
if	(object_id('Person.PersonSparse') is not null)
	begin
		drop table	Person.PersonSparse
		;
	end
go

create table	Person.PersonSparse
	(
	-- Common Attributes
	PersonID				int					identity(1, 1)	not null
	,TaxID					nvarchar(9)						not null
	-- Natural Person Attributes
	,FirstName				nvarchar(50)						sparse
	,MiddleName				nvarchar(50)						sparse
	,LastName				nvarchar(50)						sparse
	,BirthDate				date							sparse
	,SSN					nvarchar(12)						sparse
	-- Legal Person Attributes
	,FullName				nvarchar(450)						sparse
	,AbbreviatedName			nvarchar(450)						sparse
	,RegistrationDate			date							sparse
	,CompanyNumber				nvarchar(15)						sparse
	-- Distinguishing Attribute
	,IsLegalPerson				bit							not null
	-- Computed Attribute
	,FriendlyName	as
			cast(case
				when	(IsLegalPerson = cast(1 as bit))
					then	AbbreviatedName
				else	LastName
					+ N', '
					+ FirstName
					+ coalesce(N' ' + MiddleName, N'')
				end as nvarchar(450))
			persisted
	-- Sparse column set
	,PersonSet				xml		column_set for all_sparse_columns
	,constraint	pk_Person_PersonSparse
			primary key clustered
			(
			PersonID
			)
	,constraint	uq_Person_PersonSparse_TaxID
			unique nonclustered
			(
			TaxID
			,IsLegalPerson
			)
	,constraint	uq_Person_PersonSparse_FriendlyName
			unique nonclustered
			(
			FriendlyName
			)
	,constraint	chk_Person_PersonSparse_IsLegalPerson
			check
			(
			(
			(IsLegalPerson = cast(1 as bit))
			and (FirstName is null)
			and (MiddleName is null)
			and (LastName is null)
			and (BirthDate is null)
			and (SSN is null)
			and (FullName is not null)
			and (AbbreviatedName is not null)
			and (RegistrationDate is not null)
			and (CompanyNumber is not null)
			) or (
			(IsLegalPerson = cast(0 as bit))
			and (FirstName is not null)
			and (LastName is not null)
			and (BirthDate is not null)
			and (SSN is not null)
			and (FullName is null)
			and (AbbreviatedName is null)
			and (RegistrationDate is null)
			and (CompanyNumber is null)
			)
			)
	)
;
go


-- Person.PersonXml (Drop)
if	(object_id('Person.PersonXml') is not null)
	begin
		drop table	Person.PersonXml
		;
	end
go

-- Person.Attribute_Get_byLocalName
if	(object_id('Person.Attribute_Get_byLocalName') is not null)
	begin
		drop function	Person.Attribute_Get_byLocalName
		;
	end
go

-- Person.FriendlyName (Drop)
if	(object_id('Person.FriendlyName') is not null)
	begin
		drop function	Person.FriendlyName
		;
	end
go

-- Person.IsLegal (Drop)
if	(object_id('Person.IsLegal') is not null)
	begin
		drop function	Person.IsLegal
		;
	end
go

-- Person.PersonXmlSchema
if	(exists (
		select	*
			from	sys.xml_schema_collections
			where	(schema_name(xml_schema_collections.schema_id) + N'.' + xml_schema_collections.name = N'Person.PersonXmlSchema')
		))
	begin
		drop xml schema collection	Person.PersonXmlSchema
		;
	end
go

declare	@legalPersonXmlSchema			xml
declare	@naturalPersonXmlSchema		xml

set	@legalPersonXmlSchema
		= (
		select	cast(XmlFile.XmlSchemaBlob as xml)
			from	openrowset
					(
					bulk	N'C:\SupertypesAndSubtypes\XMLSchema\MiLambda.LegalPerson.xsd'
					,single_blob
					) XmlFile (XmlSchemaBlob)
		)
;

set	@naturalPersonXmlSchema
		= (
		select	cast(XmlFile.XmlSchemaBlob as xml)
			from	openrowset
					(
					bulk	N'C:\SupertypesAndSubtypes\XMLSchema\MiLambda.NaturalPerson.xsd'
					,single_blob
					) XmlFile (XmlSchemaBlob)
		)
;

create xml schema collection	Person.PersonXmlSchema
as
@legalPersonXmlSchema
;

alter xml schema collection	Person.PersonXmlSchema
add
@naturalPersonXmlSchema
;
go

-- Person.IsLegal (Create)
create function	Person.IsLegal
	(
	@Person					xml(Person.PersonXmlSchema)
	)
returns	bit
with	schemabinding
as
begin
	return	(
		case
			when	(@Person.exist
					(N'
					declare	namespace lp = "http://milambda.net/legalPerson";
					
					/lp:*
					') = 1)
				then	cast(1 as bit)
			else	cast(0 as bit)
			end
		)
	;
end
go

-- Person.FriendlyName (Create)
create function	Person.FriendlyName
	(
	@Person					xml(Person.PersonXmlSchema)
	)
returns	nvarchar(450)
with	schemabinding
as
begin
	return	(
		case
			when	(@Person.exist
					(N'
					declare	namespace lp = "http://milambda.net/legalPerson";
					
					/lp:*
					') = 1)
				then	@Person.value
						(N'
						declare	namespace lp = "http://milambda.net/legalPerson";
						
						data(/lp:legalPerson[1]/@lp:abbreviatedName)
						'
						,N'nvarchar(450)'
						)
			else	cast(@Person.value
					(N'
					declare	namespace pp = "http://milambda.net/naturalPerson";
						
					data(/pp:naturalPerson[1]/@pp:lastName)
					'
					,N'nvarchar(50)')
				+ N', '
				+ @Person.value
					(N'
					declare	namespace pp = "http://milambda.net/naturalPerson";
						
					data(/pp:naturalPerson[1]/@pp:firstName)
					'
					,N'nvarchar(50)')
				+ coalesce
					(
					@Person.value
					(N'
					declare	namespace pp = "http://milambda.net/naturalPerson";
						
					data(/pp:naturalPerson[1]/@pp:middleName)
					'
					,N'nvarchar(50)')
					,N''
					) as nvarchar(450))
			end
		)
	;
end
go

-- Person.PersonXml (Create)
create table	Person.PersonXml
	(
	-- Common Attributes
	PersonID				int					identity(1, 1)	not null
	,TaxID					nvarchar(9)						not null
	-- Natural or Legal Person Entity
	,Person					xml(Person.PersonXmlSchema)				not null
	-- Computed Attributes
	,IsLegalPerson	as
			(
			Person.IsLegal(Person)
			)
			persisted
	,FriendlyName	as
			(
			Person.FriendlyName(Person)
			)
			persisted
	,constraint	pk_Person_PersonXml
			primary key clustered
			(
			PersonID
			)
	,constraint	uq_Person_PersonXml_TaxID
			unique nonclustered
			(
			TaxID
			,IsLegalPerson
			)
	,constraint	uq_Person_PersonXml_FriendlyName
			unique nonclustered
			(
			FriendlyName
			)
	)
;
go


-- Person.PersonJson
if	(object_id('Person.PersonJson') is not null)
	begin
		drop table	Person.PersonJson
		;
	end
go

create table	Person.PersonJson
	(
	-- Common Attributes
	PersonID				int					identity(1, 1)	not null
	,TaxID					nvarchar(9)						not null
	-- Natural or Legal Person Entity
	,Person					nvarchar(max)						not null
	-- Computed Attributes
	,IsLegalPerson	as
			cast(json_value(Person, N'$.IsLegalPerson') as bit)
			persisted
	,FriendlyName	as
			cast(case
				when	(cast(json_value(Person, N'$.IsLegalPerson') as bit) = cast(1 as bit))
					then	cast(json_value(Person, N'$.AbbreviatedName') as nvarchar(450))
				else	cast(json_value(Person, N'$.LastName') as nvarchar(50))
					+ N', '
					+ cast(json_value(Person, N'$.FirstName') as nvarchar(50))
					+ coalesce(N' ' + cast(json_value(Person, N'$.MiddleName') as nvarchar(50)), N'')
				end as nvarchar(450))
			persisted
	,constraint	pk_Person_PersonJson
			primary key clustered
			(
			PersonID
			)
	,constraint	uq_Person_PersonJson_TaxID
			unique nonclustered
			(
			TaxID
			,IsLegalPerson
			)
	,constraint	uq_Person_PersonJson_FriendlyName
			unique nonclustered
			(
			FriendlyName
			)
	)
;
go


-- Person.PersonClr (Drop)
if	(object_id('Person.PersonClr') is not null)
	begin
		drop table	Person.PersonClr
		;
	end
go

-- Common.NaturalPersonClr (Drop)
if	(type_id('Common.NaturalPersonClr') is not null)
	begin
		drop type	Common.NaturalPersonClr
		;
	end
go

-- Common.LegalPersonClr (Drop)
if	(type_id('Common.LegalPersonClr') is not null)
	begin
		drop type	Common.LegalPersonClr
		;
	end
go

-- MiLambda.Person.SqlClr (Assembly)
if	(exists (
		select	*
			from	sys.assemblies
			where	(assemblies.[name] = N'MiLambda.Person.SqlClr')
		))
	begin
		drop assembly	[MiLambda.Person.SqlClr]
		;
	end
go

create assembly	[MiLambda.Person.SqlClr]
	authorization	dbo
	from	N'C:\SQLCLR\MiLambda\MiLambda.Person.SqlClr.dll'
	with	permission_set = safe
;
go

-- Common.LegalPersonClr (Create)
create type	Common.LegalPersonClr
external name	[MiLambda.Person.SqlClr].[MiLambda.Person.LegalPerson]
;
go

-- Common.NaturalPersonClr (Create)
create type	Common.NaturalPersonClr
external name	[MiLambda.Person.SqlClr].[MiLambda.Person.NaturalPerson]
;
go

-- Person.PersonClr (Create)
create table	Person.PersonClr
	(
	-- Common Attributes
	PersonID				int					identity(1, 1)	not null
	,TaxID					nvarchar(9)						not null
	-- Natural Person Entity
	,NaturalPerson				Common.NaturalPersonClr				null
	-- Legal Person Entity
	,LegalPerson				Common.LegalPersonClr					null
	-- Computed Attributes
	,IsLegalPerson	as
			(
			case
				when	(LegalPerson is null)
					then	cast(0 as bit)
				else	cast(1 as bit)
				end
			)
			persisted
	,FriendlyName	as
			(
			cast(case
				when	(LegalPerson is null)
					then	NaturalPerson.FriendlyName()
				else	LegalPerson.FriendlyName()
				end as nvarchar(450))
			)
			persisted
	,constraint	pk_Person_PersonClr
			primary key clustered
			(
			PersonID
			)
	,constraint	uq_Person_PersonClr_TaxID
			unique nonclustered
			(
			TaxID
			,IsLegalPerson
			)
	,constraint	uq_Person_PersonClr_FriendlyName
			unique nonclustered
			(
			FriendlyName
			)
	,constraint	chk_Person_PersonClr_Person
			check
			(
			(
			(LegalPerson is null)
			and (NaturalPerson is not null)
			) or (
			(LegalPerson is not null)
			and (NaturalPerson is null)
			)
			)
	)
;
go
