-- SupertypesAndSubtypes_Data_Query
use	Super
;
go

-- Prepare temporary data
create table	#BirthData
	(
	BusinessEntityID			int							not null
	,BirthDate				date							not null
	,primary key clustered
		(
		BusinessEntityID
		)
	)
;
create table	#RegistryData
	(
	BusinessEntityID			int							not null
	,RegistrationDate			date							not null
	,rowguid				uniqueidentifier					not null
		default
		(
		newid()
		)
	,primary key clustered
		(
		BusinessEntityID
		)
	)
;

-- Birth Data
insert	#BirthData
	(
	BusinessEntityID
	,BirthDate
	)
	select	Person.BusinessEntityID as BusinessEntityID
		,cast(dateadd(year, -Subtraction.Subtrahend, Person.ModifiedDate) as date) as BirthDate
		from	AdventureWorks2014.Person.Person
			cross apply	(
					select	top(1)
						Subtraction.Subtrahend as Subtrahend
						from	(
							values
							(10)
							,(15)
							,(20)
							,(25)
							,(30)
							,(35)
							,(40)
							) Subtraction (Subtrahend)
						order by	newid()
					) Subtraction
		where	(Person.PersonType = N'SC')
;

-- Registry Data
insert	#RegistryData
	(
	BusinessEntityID
	,RegistrationDate
	)
	select	Vendor.BusinessEntityID as BusinessEntityID
		,cast(dateadd(year, -Subtraction.Subtrahend, Vendor.ModifiedDate) as date) as RegistrationDate
		from	AdventureWorks2014.Purchasing.Vendor
			cross apply	(
					select	top(1)
						Subtraction.Subtrahend as Subtrahend
						from	(
							values
							(20)
							,(25)
							,(30)
							,(35)
							,(40)
							,(45)
							,(50)
							) Subtraction (Subtrahend)
						order by	newid()
					) Subtraction
;


-- Person.PersonSparse (Data)
truncate table	Person.PersonSparse
;
insert	Person.PersonSparse
	(
	TaxID
	,FirstName
	,MiddleName
	,LastName
	,BirthDate
	,SSN
	,FullName
	,AbbreviatedName
	,RegistrationDate
	,CompanyNumber
	,IsLegalPerson
	)
	select	left(cast(cast(cast(Person.rowguid as varbinary(32)) as bigint) * -1 as nvarchar(64)), 9) as TaxID
		,Person.FirstName as FirstName
		,Person.MiddleName as MiddleName
		,Person.LastName as LastName
		,BirthData.BirthDate as BirthDate
		,right(cast(cast(cast(Person.rowguid as varbinary(32)) as bigint) * -1 as nvarchar(64)), 12) as SSN
		,cast(null as nvarchar(450)) as FullName
		,cast(null as nvarchar(450)) as AbbreviatedName
		,cast(null as date) as RegistrationDate
		,cast(null as nvarchar(15)) as CompanyNumber
		,cast(0 as bit) as IsLegalPerson
		from	AdventureWorks2014.Person.Person
			inner join	#BirthData BirthData
					on	BirthData.BusinessEntityID = Person.BusinessEntityID
	union
	select	left(cast(cast(cast(RegistryData.rowguid as varbinary(32)) as bigint) * -1 as nvarchar(64)), 9)
		,cast(null as nvarchar(50))
		,cast(null as nvarchar(50))
		,cast(null as nvarchar(50))
		,cast(null as date)
		,cast(null as nvarchar(12))
		,Vendor.Name
		,case
			when	(charindex(N',', Vendor.Name) > 0)
				then	left(Vendor.Name, charindex(N',', Vendor.Name) - 1)
			else	Vendor.Name
			end
		,RegistryData.RegistrationDate as RegistrationDate
		,right(cast(cast(cast(RegistryData.rowguid as varbinary(32)) as bigint) * -1 as nvarchar(64)), 15)
		,cast(1 as bit)
		from	AdventureWorks2014.Purchasing.Vendor
			inner join	#RegistryData RegistryData
					on	RegistryData.BusinessEntityID = Vendor.BusinessEntityID
;
-- Person.PersonSparse (Check)
select	*
	from	Person.PersonSparse
;


-- Person.PersonJson (Data)
truncate table	Person.PersonJson
;
begin
	with	PersonJson
	as
	(
	select	left(cast(cast(cast(Person.rowguid as varbinary(32)) as bigint) * -1 as nvarchar(64)), 9) as TaxID
		,Person
			= json_query
				(
				(
				select	PersonData.FirstName as FirstName
					,PersonData.MiddleName as MiddleName
					,PersonData.LastName as LastName
					,BirthData.BirthDate as BirthDate
					,right(cast(cast(cast(PersonData.rowguid as varbinary(32)) as bigint) * -1 as nvarchar(64)), 12) as SSN
					,cast(0 as bit) as IsLegalPerson
					from	AdventureWorks2014.Person.Person PersonData
						inner join	#BirthData BirthData
								on	BirthData.BusinessEntityID = PersonData.BusinessEntityID
					where	(PersonData.BusinessEntityID = Person.BusinessEntityID)
					for json path
				)
				,N'$[0]'
				)
		from	AdventureWorks2014.Person.Person
	union
	select	left(cast(cast(cast(MainRegistryData.rowguid as varbinary(32)) as bigint) * -1 as nvarchar(64)), 9) as TaxID
		,Person
			= json_query
				(
				(
				select	PersonData.Name as FullName
					,AbbreviatedName
						= case
							when	(charindex(N',', PersonData.Name) > 0)
								then	left(PersonData.Name, charindex(N',', PersonData.Name) - 1)
							else	PersonData.Name
							end
					,RegistryData.RegistrationDate as RegistrationDate
					,right(cast(cast(cast(RegistryData.rowguid as varbinary(32)) as bigint) * -1 as nvarchar(64)), 15) as CompanyNumber
					,cast(1 as bit) as IsLegalPerson
					from	AdventureWorks2014.Purchasing.Vendor PersonData
						inner join	#RegistryData RegistryData
								on	RegistryData.BusinessEntityID = Vendor.BusinessEntityID
					where	(PersonData.BusinessEntityID = Vendor.BusinessEntityID)
					for json path
				)
				,N'$[0]'
				)
		from	AdventureWorks2014.Purchasing.Vendor
			inner join	#RegistryData MainRegistryData
					on	MainRegistryData.BusinessEntityID = Vendor.BusinessEntityID
	)
	insert	Person.PersonJson
		(
		TaxID
		,Person
		)
		select	PersonJson.TaxID as TaxID
			,PersonJson.Person as Person
			from	PersonJson
			where	(PersonJson.Person is not null)
end
;
-- Person.PersonJson (Check)
select	*
	from	Person.PersonJson
;


-- Person.PersonXml (Data)
truncate table	Person.PersonXml
;
-- Natural Person
begin
	with	xmlnamespaces
		(
		N'http://milambda.net/naturalPerson' as pp
		)
	,PersonXml
	as
	(
	select	left(cast(cast(cast(Person.rowguid as varbinary(32)) as bigint) * -1 as nvarchar(64)), 9) as TaxID
		,Person
			= (
			select	PersonData.FirstName as [@pp:firstName]
				,PersonData.MiddleName as [@pp:middleName]
				,PersonData.LastName as [@pp:lastName]
				,BirthData.BirthDate as [@pp:birthDate]
				,right(cast(cast(cast(PersonData.rowguid as varbinary(32)) as bigint) * -1 as nvarchar(64)), 12) as [@pp:ssn]
				from	AdventureWorks2014.Person.Person PersonData
					inner join	#BirthData BirthData
							on	BirthData.BusinessEntityID = PersonData.BusinessEntityID
				where	(PersonData.BusinessEntityID = Person.BusinessEntityID)
				for xml path(N'pp:naturalPerson'), type
			)
		from	AdventureWorks2014.Person.Person
	)
	insert	Person.PersonXml
		(
		TaxID
		,Person
		)
		select	PersonXml.TaxID as TaxID
			,cast(PersonXml.Person as xml) as Person
			--,Person.IsLegal(Person)
			from	PersonXml
			where	(PersonXml.Person is not null)
	;
end

-- Legal Person
begin
	with	xmlnamespaces
		(
		N'http://milambda.net/legalPerson' as lp
		)
	,PersonXml
	as
	(
	select	left(cast(cast(cast(MainRegistryData.rowguid as varbinary(32)) as bigint) * -1 as nvarchar(64)), 9) as TaxId
		,Person
			= (
			select	PersonData.Name as [@lp:fullName]
				,PersonData.Name as [@lp:abbreviatedName]
				,RegistryData.RegistrationDate as [@lp:registrationDate]
				,right(cast(cast(cast(RegistryData.rowguid as varbinary(32)) as bigint) * -1 as nvarchar(64)), 15) as [@lp:companyNumber]
				from	AdventureWorks2014.Purchasing.Vendor PersonData
					inner join	#RegistryData RegistryData
							on	RegistryData.BusinessEntityID = Vendor.BusinessEntityID
				where	(PersonData.BusinessEntityID = Vendor.BusinessEntityID)
				for xml path(N'lp:legalPerson'), type
			)
		from	AdventureWorks2014.Purchasing.Vendor
			inner join	#RegistryData MainRegistryData
					on	MainRegistryData.BusinessEntityID = Vendor.BusinessEntityID
	)
	insert	Person.PersonXml
		(
		TaxID
		,Person
		)
		select	PersonXml.TaxID as TaxID
			,cast(PersonXml.Person as xml) as Person
			from	PersonXml
			where	(PersonXml.Person is not null)
	;
end
-- Person.PersonXml (Check)
select	*
	from	Person.PersonXml
;


-- Person.PersonClr (Data)
truncate table	Person.PersonClr
;
insert	Person.PersonClr
	(
	TaxID
	,NaturalPerson
	,LegalPerson
	)
	select	PersonXml.TaxID as TaxID
		,cast(cast(PersonXml.Person as nvarchar(max)) as Common.NaturalPersonClr) as NaturalPerson
		,null as LegalPerson
		from	Person.PersonXml
		where	(PersonXml.IsLegalPerson = cast(0 as bit))
	union all
	select	PersonXml.TaxID
		,null
		,cast(cast(PersonXml.Person as nvarchar(max)) as Common.LegalPersonClr)
		from	Person.PersonXml
		where	(PersonXml.IsLegalPerson = cast(1 as bit))
;
-- Person.PersonClr (Check)
select	*
	from	Person.PersonClr
;
go

-- Remove temporary data
drop table	#BirthData
;
drop table	#RegistryData
;
go
