-- SupertypesAndSubtypes_Person_PersonClr_Examples
use	Super
;
go

select	*
	from	Person.PersonClr
;

select	PersonId
	,TaxID
	,cast(NaturalPerson.FirstName as nvarchar(50)) as FirstName
	,cast(NaturalPerson.MiddleName as nvarchar(50)) as MiddleName
	,cast(NaturalPerson.LastName as nvarchar(50)) as LastName
	,cast(NaturalPerson.BirthDate as date) as BirthDate
	,cast(NaturalPerson.SSN as nvarchar(12)) as SSN
	,cast(LegalPerson.FullName as nvarchar(450)) as FullName
	,cast(LegalPerson.AbbreviatedName as nvarchar(450)) as AbbreviatedName
	,cast(LegalPerson.RegistrationDate as date) as RegistrationDate
	,cast(LegalPerson.CompanyNumber as nvarchar(15)) as CompanyNumber
	,IsLegalPerson
	,FriendlyName
	from	Person.PersonClr
;

select	PersonId
	,TaxID
	,cast(LegalPerson.FullName as nvarchar(450)) as FullName
	,cast(LegalPerson.AbbreviatedName as nvarchar(450)) as AbbreviatedName
	,cast(LegalPerson.RegistrationDate as date) as RegistrationDate
	,cast(LegalPerson.CompanyNumber as nvarchar(15)) as CompanyNumber
	,FriendlyName
	from	Person.PersonClr
	where	(IsLegalPerson = cast(1 as bit))
;

select	PersonId
	,TaxID
	,cast(NaturalPerson.FirstName as nvarchar(50)) as FirstName
	,cast(NaturalPerson.MiddleName as nvarchar(50)) as MiddleName
	,cast(NaturalPerson.LastName as nvarchar(50)) as LastName
	,cast(NaturalPerson.BirthDate as date) as BirthDate
	,cast(NaturalPerson.SSN as nvarchar(12)) as SSN
	,FriendlyName
	from	Person.PersonClr
	where	(IsLegalPerson = cast(0 as bit))
;


-- Unique CompanyNumber
alter table	Person.PersonClr
	add	CompanyNumber	as
				coalesce
					(
					-- Actual CompanyNumber, if exists; ...
					cast(LegalPerson.CompanyNumberIndex() as nvarchar(15))
					-- ... otherwise, primary key.
					,cast(PersonID * -1 as nvarchar(15))
					)
				persisted
;
go

select	CompanyNumber
	,IsLegalPerson
	from	Person.PersonClr
;

create unique nonclustered index	ux_Person_PersonClr_CompanyNumber
	on	Person.PersonClr
		(
		CompanyNumber
		,IsLegalPerson
		)
;
go


-- Unique SSN
alter table	Person.PersonClr
	add	SSN		as
				coalesce
					(
					-- Actual SSN, if exists; ...
					cast(NaturalPerson.SSNIndex() as nvarchar(15))
					-- ... otherwise, primary key.
					,cast(PersonID * -1 as nvarchar(15))
					)
				persisted
;
go

select	SSN
	,IsLegalPerson
	from	Person.PersonClr
;

create unique nonclustered index	ux_Person_PersonClr_SSN
	on	Person.PersonClr
		(
		SSN
		,IsLegalPerson
		)
;
go


-- Indexes on computed columns (include Actual Execution Plan)
declare	@ssn					nvarchar(12)
declare	@companyNumber				nvarchar(15)

set	@ssn
		= (
		select	top(1)
			SSN
			from	Person.PersonClr
			where	(SSN not like N'-%')
			order by	newid()
		)
;
set	@companyNumber
		= (
		select	top(1)
			CompanyNumber
			from	Person.PersonClr
			where	(CompanyNumber not like N'-%')
			order by	newid()
		)
;

set	statistics io	on
;
select	PersonID
	,TaxID
	,NaturalPerson
	,IsLegalPerson
	,FriendlyName
	from	Person.PersonClr
	where	(SSN = @ssn)
		and (IsLegalPerson = cast(0 as bit))
;
select	PersonID
	,TaxID
	,LegalPerson
	,IsLegalPerson
	,FriendlyName
	from	Person.PersonClr
	where	(CompanyNumber = @companyNumber)
		and (IsLegalPerson = cast(1 as bit))
;
set	statistics io	off
;
go


-- No nonclustered index (include Actual Execution Plan)
set	statistics io	on
;
select	*
	from	Person.PersonClr
	where	(NaturalPerson.LastName = N'Kim')
;
select	*
	from	Person.PersonClr
	where	(LegalPerson.AbbreviatedName = N'Carlson Specialties')
;
set	statistics io	off
;
go



/* Clean up
drop index	ux_Person_PersonClr_CompanyNumber
	on	Person.PersonClr
;
go
alter table	Person.PersonClr
	drop column	CompanyNumber
;
go

drop index	ux_Person_PersonClr_SSN
	on	Person.PersonClr
;
go
alter table	Person.PersonClr
	drop column	SSN
;
go
*/
