-- SupertypesAndSubtypes_Person_PersonJson_Examples
use	Super
;
go

select	*
	from	Person.PersonJson
;

select	PersonId
	,TaxID
	,cast(json_value(Person, N'$.FirstName') as nvarchar(50)) as FirstName
	,cast(json_value(Person, N'$.MiddleName') as nvarchar(50)) as MiddleName
	,cast(json_value(Person, N'$.LastName') as nvarchar(50)) as LastName
	,cast(json_value(Person, N'$.BirthDate') as date) as BirthDate
	,cast(json_value(Person, N'$.SSN') as nvarchar(12)) as SSN
	,cast(json_value(Person, N'$.FullName') as nvarchar(450)) as FullName
	,cast(json_value(Person, N'$.AbbreviatedName') as nvarchar(450)) as AbbreviatedName
	,cast(json_value(Person, N'$.RegistrationDate') as date) as RegistrationDate
	,cast(json_value(Person, N'$.CompanyNumber') as nvarchar(15)) as CompanyNumber
	,IsLegalPerson
	,FriendlyName
	from	Person.PersonJson
;

select	PersonId
	,TaxID
	,cast(json_value(Person, N'$.FullName') as nvarchar(450)) as FullName
	,cast(json_value(Person, N'$.AbbreviatedName') as nvarchar(450)) as AbbreviatedName
	,cast(json_value(Person, N'$.RegistrationDate') as date) as RegistrationDate
	,cast(json_value(Person, N'$.CompanyNumber') as nvarchar(15)) as CompanyNumber
	,FriendlyName
	from	Person.PersonJson
	where	(IsLegalPerson = cast(1 as bit))
;

select	PersonId
	,TaxID
	,cast(json_value(Person, N'$.FirstName') as nvarchar(50)) as FirstName
	,cast(json_value(Person, N'$.MiddleName') as nvarchar(50)) as MiddleName
	,cast(json_value(Person, N'$.LastName') as nvarchar(50)) as LastName
	,cast(json_value(Person, N'$.BirthDate') as date) as BirthDate
	,cast(json_value(Person, N'$.SSN') as nvarchar(12)) as SSN
	,FriendlyName
	from	Person.PersonJson
	where	(IsLegalPerson = cast(0 as bit))
;


-- Unique CompanyNumber
alter table	Person.PersonJson
	add	CompanyNumber	as
				coalesce
					(
					-- Actual CompanyNumber, if exists; ...
					cast(json_value(Person, N'$.CompanyNumber') as nvarchar(15))
					-- ... otherwise, primary key.
					,cast(PersonID * -1 as nvarchar(15))
					)
				--persisted
;
go

select	CompanyNumber
	,IsLegalPerson
	from	Person.PersonJson
;

create unique nonclustered index	ux_Person_PersonJson_CompanyNumber
	on	Person.PersonJson
		(
		CompanyNumber
		,IsLegalPerson
		)
;
go

-- Unique SSN
alter table	Person.PersonJson
	add	SSN		as
				coalesce
					(
					-- Actual SSN, if exists; ...
					cast(json_value(Person, N'$.SSN') as nvarchar(12))
					-- ... otherwise, primary key.
					,cast(PersonID * -1 as nvarchar(12))
					)
				--persisted
;
go

select	SSN
	,IsLegalPerson
	from	Person.PersonJson
;

create unique nonclustered index	ux_Person_PersonJson_SSN
	on	Person.PersonJson
		(
		SSN
		,IsLegalPerson
		)
;
go


-- Indexes on computed columns (include Actual Execution Plan)
declare	@ssn					nvarchar(12)
declare	@companyNumber				nvarchar(15)

set	@ssn
		= (
		select	top(1)
			SSN
			from	Person.PersonJson
			where	(SSN not like N'-%')
			order by	newid()
		)
;
set	@companyNumber
		= (
		select	top(1)
			CompanyNumber
			from	Person.PersonJson
			where	(CompanyNumber not like N'-%')
			order by	newid()
		)
;

set	statistics io	on
;
select	PersonID
	,TaxID
	,Person
	,IsLegalPerson
	,FriendlyName
	from	Person.PersonJson
	where	(SSN = @ssn)
		and (IsLegalPerson = cast(0 as bit))
;
select	PersonID
	,TaxID
	,Person
	,IsLegalPerson
	,FriendlyName
	from	Person.PersonJson
	where	(CompanyNumber = @companyNumber)
		and (IsLegalPerson = cast(1 as bit))
;
set	statistics io	off
;
go


-- No nonclustered index (include Actual Execution Plan)
set	statistics io	on
;
select	*
	from	Person.PersonJson
	where	(cast(json_value(Person, N'$.LastName') as nvarchar(50)) = N'Kim')
;
select	*
	from	Person.PersonJson
	where	(cast(json_value(Person, N'$.AbbreviatedName') as nvarchar(450)) = N'Carlson Specialties')
;
set	statistics io	off
;
go


/* Clean up
drop index	ux_Person_PersonJson_CompanyNumber
	on	Person.PersonJson
;
go
alter table	Person.PersonJson
	drop column	CompanyNumber
;
go
drop index	ux_Person_PersonJson_SSN
	on	Person.PersonJson
;
go
alter table	Person.PersonJson
	drop column	SSN
;
go
*/
