-- SupertypesAndSubtypes_Person_PersonSparse_Examples
use	Super
;
go

select	*
	from	Person.PersonSparse
;

select	PersonId
	,TaxID
	,FirstName
	,MiddleName
	,LastName
	,BirthDate
	,SSN
	,FullName
	,AbbreviatedName
	,RegistrationDate
	,CompanyNumber
	,IsLegalPerson
	,FriendlyName
	from	Person.PersonSparse
;

select	PersonId
	,TaxID
	,FullName
	,AbbreviatedName
	,RegistrationDate
	,CompanyNumber
	,FriendlyName
	from	Person.PersonSparse
	where	(IsLegalPerson = cast(1 as bit))
;

select	PersonId
	,TaxID
	,FirstName
	,MiddleName
	,LastName
	,BirthDate
	,SSN
	,FriendlyName
	from	Person.PersonSparse
	where	(IsLegalPerson = cast(0 as bit))
;


-- Unique CompanyNumber
create unique nonclustered index	ux_Person_PersonSparse_CompanyNumber
	on	Person.PersonSparse
		(
		CompanyNumber
		)
	where	(CompanyNumber is not null)
;
go

-- Unique SSN
create unique nonclustered index	ux_Person_PersonSparse_SSN
	on	Person.PersonSparse
		(
		SSN
		)
	where	(SSN is not null)
;
go


-- Filtered indexes (include Actual Execution Plan)
declare	@ssn					nvarchar(12)
declare	@companyNumber				nvarchar(15)

set	@ssn
		= (
		select	top(1)
			SSN
			from	Person.PersonSparse
			where	(SSN is not null)
			order by	newid()
		)
;
set	@companyNumber
		= (
		select	top(1)
			CompanyNumber
			from	Person.PersonSparse
			where	(CompanyNumber is not null)
			order by	newid()
		)
;

set	statistics io	on
;
select	*
	from	Person.PersonSparse
	where	(SSN = @ssn)
;
select	*
	from	Person.PersonSparse
	where	(CompanyNumber = @companyNumber)
;
set	statistics io	off
;
go


-- No nonclustered index (include Actual Execution Plan)
set	statistics io	on
;
select	*
	from	Person.PersonSparse
	where	(LastName = N'Kim')
;
select	*
	from	Person.PersonSparse
	where	(AbbreviatedName = N'Carlson Specialties')
;
set	statistics io	off
;
go



/* Clean up
drop index	ux_Person_PersonSparse_CompanyNumber
	on	Person.PersonSparse
;
go
drop index	ux_Person_PersonSparse_SSN
	on	Person.PersonSparse
;
go
*/
