-- SupertypesAndSubtypes_Person_PersonXml_Examples
use	Super
;
go

select	*
	from	Person.PersonXml
;

begin
	with	xmlnamespaces
		(
		N'http://milambda.net/legalPerson' as lp
		)
	select	PersonID
		,TaxID
		,Person.value(N'/lp:legalPerson[1]/@lp:fullName', N'nvarchar(450)') as FullName
		,Person.value(N'/lp:legalPerson[1]/@lp:abbreviatedName', N'nvarchar(450)') as AbbreviatedName
		,Person.value(N'/lp:legalPerson[1]/@lp:registrationDate', N'date') as RegistrationDate
		,Person.value(N'/lp:legalPerson[1]/@lp:companyNumber', N'nvarchar(15)') as CompanyNumber
		,FriendlyName
		from	Person.PersonXml
		where	(IsLegalPerson = cast(1 as bit))
	;
end

begin
	with	xmlnamespaces
		(
		N'http://milambda.net/naturalPerson' as pp
		)
	select	PersonID
		,TaxID
		,Person.value(N'/pp:naturalPerson[1]/@pp:firstName', N'nvarchar(450)') as FirstName
		,Person.value(N'/pp:naturalPerson[1]/@pp:middleName', N'nvarchar(450)') as MiddleName
		,Person.value(N'/pp:naturalPerson[1]/@pp:lastName', N'nvarchar(450)') as LastName
		,Person.value(N'/pp:naturalPerson[1]/@pp:birthDate', N'date') as BirthDate
		,Person.value(N'/pp:naturalPerson[1]/@pp:ssn', N'nvarchar(12)') as SSN
		,FriendlyName
		from	Person.PersonXml
		where	(IsLegalPerson = cast(0 as bit))
	;
end


-- Person.Attribute_Get_byLocalName
if	(object_id('Person.Attribute_Get_byLocalName') is not null)
	begin
		drop function	Person.Attribute_Get_byLocalName
		;
	end
go

create function	Person.Attribute_Get_byLocalName
	(
	@Person					xml(Person.PersonXmlSchema)
	,@LocalName				sysname
	)
returns	nvarchar(450)
with	schemabinding
as
begin
	return	(
		nullif
			(
			@Person.value
				(
				N'string(/*[1]/attribute::*[fn:local-name() = sql:variable("@LocalName")][1])'
				,N'nvarchar(450)'
				)
			,N''
			)
		)
	;
end
go


-- Unique CompanyNumber
alter table	Person.PersonXml
	add	CompanyNumber	as
				(
				coalesce
					(
					cast(Person.Attribute_Get_byLocalName
						(
						Person
						,N'companyNumber'
						) as nvarchar(15))
					,cast(PersonId * -1 as nvarchar(15))
					)
				)
;
go

select	CompanyNumber
	,IsLegalPerson
	from	Person.PersonXml
;

create unique nonclustered index	ux_Person_PersonXml_CompanyNumber
	on	Person.PersonXml
		(
		CompanyNumber
		,IsLegalPerson
		)
;
go

-- Unique SSN
alter table	Person.PersonXml
	add	SSN		as
				(
				coalesce
					(
					cast(Person.Attribute_Get_byLocalName
						(
						Person
						,N'ssn'
						) as nvarchar(12))
					,cast(PersonId * -1 as nvarchar(12))
					)
				)
;
go

select	SSN
	,IsLegalPerson
	from	Person.PersonXml
;

create unique nonclustered index	ux_Person_PersonXml_SSN
	on	Person.PersonXml
		(
		SSN
		,IsLegalPerson
		)
;
go


-- Indexes on computed columns (include Actual Execution Plan)
declare	@ssn					nvarchar(12)
declare	@companyNumber				nvarchar(15)

set	@ssn
		= (
		select	top(1)
			SSN
			from	Person.PersonXml
			where	(SSN not like N'-%')
			order by	newid()
		)
;
set	@companyNumber
		= (
		select	top(1)
			CompanyNumber
			from	Person.PersonXml
			where	(CompanyNumber not like N'-%')
			order by	newid()
		)
;

set	statistics io	on
;
select	PersonID
	,TaxID
	,Person
	,IsLegalPerson
	,FriendlyName
	from	Person.PersonXml
	where	(SSN = @ssn)
		and (IsLegalPerson = cast(0 as bit))
;
select	PersonID
	,TaxID
	,Person
	,IsLegalPerson
	,FriendlyName
	from	Person.PersonXml
	where	(CompanyNumber = @companyNumber)
		and (IsLegalPerson = cast(1 as bit))
;
set	statistics io	off
;
go


-- No nonclustered index (include Actual Execution Plan)
set	statistics io	on
;
select	*
	from	Person.PersonXml
	where	(Person.Attribute_Get_byLocalName(Person, N'lastName') = N'Kim')
;
select	*
	from	Person.PersonXml
	where	(Person.Attribute_Get_byLocalName(Person, N'abbreviatedName') = N'Carlson Specialties')
;
set	statistics io	off
;
go


-- Selective XML indexes
create selective xml index	sxx_Person_PersonXml_Person
	on	Person.PersonXml
		(
		Person
		)
	with	xmlnamespaces
		(
		N'http://milambda.net/legalPerson' as lp
		,N'http://milambda.net/naturalPerson' as pp
		)
	for	(
		legalPersonPath = N'lp:legalPerson' as xquery N'node()'
		,naturalPersonPath = N'/pp:naturalPerson' as xquery N'node()'
		-- XML Schema Data Type Paths
		,fullNamePath = '/lp:legalPerson/@lp:fullName' as xquery singleton
		,abbreviatedNamePath = '/lp:legalPerson/@lp:abbreviatedName' as xquery singleton
		,registrationDatePath = '/lp:legalPerson/@lp:registrationDate' as xquery singleton
		,companyNumberPath = '/lp:legalPerson/@lp:companyNumber' as xquery singleton
		,firstNamePath = '/pp:naturalPerson/@pp:firstName' as xquery singleton
		,middleNamePath = '/pp:naturalPerson/@pp:middleName' as xquery singleton
		,lastNamePath = '/pp:naturalPerson/@pp:lastName' as xquery singleton
		,birthDatePath = '/pp:naturalPerson/@pp:birthDate' as xquery singleton
		,ssnPath = '/pp:naturalPerson/@pp:ssn' as xquery singleton
		-- SQL Server Data Type Paths
		,fullNamePathSql = '/lp:legalPerson/@lp:fullName' as sql nvarchar(450) singleton
		,abbreviatedNamePathSql = '/lp:legalPerson/@lp:abbreviatedName' as sql nvarchar(450) singleton
		,registrationDatePathSql = '/lp:legalPerson/@lp:registrationDate' as sql nvarchar(10) singleton	-- date/time not allowed
		,companyNumberPathSql = '/lp:legalPerson/@lp:companyNumber' as sql nvarchar(15) singleton
		,firstNamePathSql = '/pp:naturalPerson/@pp:firstName' as sql nvarchar(50) singleton
		,middleNamePathSql = '/pp:naturalPerson/@pp:middleName' as sql nvarchar(50) singleton
		,lastNamePathSql = '/pp:naturalPerson/@pp:lastName' as sql nvarchar(50) singleton
		,birthDatePathSql = '/pp:naturalPerson/@pp:birthDate' as sql nvarchar(10) singleton		-- date/time not allowed
		,ssnPathSql = '/pp:naturalPerson/@pp:ssn' as sql nvarchar(12) singleton
		)
;
go

create xml index	fxx_Person_PersonXml_Person_fullNamePath
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		fullNamePath
		)
;
go

create xml index	fxx_Person_PersonXml_Person_abbreviatedNamePath
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		abbreviatedNamePath
		)
;
go

create xml index	fxx_Person_PersonXml_Person_registrationDatePath
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		registrationDatePath
		)
;
go

create xml index	fxx_Person_PersonXml_Person_companyNumberPath
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		companyNumberPath
		)
;
go

create xml index	fxx_Person_PersonXml_Person_firstNamePath
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		firstNamePath
		)
;
go

create xml index	fxx_Person_PersonXml_Person_middleNamePath
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		middleNamePath
		)
;
go

create xml index	fxx_Person_PersonXml_Person_lastNamePath
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		lastNamePath
		)
;
go

create xml index	fxx_Person_PersonXml_Person_birthDatePath
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		birthDatePath
		)
;
go

create xml index	fxx_Person_PersonXml_Person_ssnPath
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		ssnPath
		)
;
go

create xml index	fxx_Person_PersonXml_Person_fullNamePathSql
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		fullNamePathSql
		)
;
go

create xml index	fxx_Person_PersonXml_Person_abbreviatedNamePathSql
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		abbreviatedNamePathSql
		)
;
go

create xml index	fxx_Person_PersonXml_Person_registrationDatePathSql
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		registrationDatePathSql
		)
;
go

create xml index	fxx_Person_PersonXml_Person_companyNumberPathSql
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		companyNumberPathSql
		)
;
go

create xml index	fxx_Person_PersonXml_Person_firstNamePathSql
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		firstNamePathSql
		)
;
go

create xml index	fxx_Person_PersonXml_Person_middleNamePathSql
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		middleNamePathSql
		)
;
go

create xml index	fxx_Person_PersonXml_Person_lastNamePathSql
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		lastNamePathSql
		)
;
go

create xml index	fxx_Person_PersonXml_Person_birthDatePathSql
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		birthDatePathSql
		)
;
go

create xml index	fxx_Person_PersonXml_Person_ssnPathSql
	on	Person.PersonXml
		(
		Person
		)
	using xml index	sxx_Person_PersonXml_Person
	for	(
		ssnPathSql
		)
;
go


-- Selective XML index, T-SQL query, scalar function
set	statistics io	on
;
select	*
	from	Person.PersonXml
	where	(Person.Attribute_Get_byLocalName(Person, N'lastName') = N'Kim')
;
select	*
	from	Person.PersonXml
	where	(Person.Attribute_Get_byLocalName(Person, N'abbreviatedName') = N'Carlson Specialties')
;
set	statistics io	off
;
go


-- Selective XML index, T-SQL query w/ .value()
set	statistics io	on
;
begin
	with	xmlnamespaces
		(
		N'http://milambda.net/naturalPerson' as pp
		)
	select	*
		from	Person.PersonXml
		where	(Person.value(N'/pp:naturalPerson[1]/@pp:lastName', N'nvarchar(50)') = N'Kim')
	;
end

begin
	with	xmlnamespaces
		(
		N'http://milambda.net/legalPerson' as lp
		)
	select	*
		from	Person.PersonXml
		where	(Person.value(N'/lp:legalPerson[1]/@lp:abbreviatedName', N'nvarchar(450)') = N'Carlson Specialties')
	;
end

set	statistics io	off
;
go


-- Selective XML index, T-SQL query w/ .exist()
set	statistics io	on
;
declare	@LastName				nvarchar(50)

set	@LastName = N'Kim'

begin
	with	xmlnamespaces
		(
		N'http://milambda.net/naturalPerson' as pp
		)
	select	*
		from	Person.PersonXml
		where	(Person.exist(N'/pp:naturalPerson[1][@pp:lastName = sql:variable("@LastName")]') = 1)
	;
end

declare	@AbbreviatedName				nvarchar(50)

set	@AbbreviatedName = N'Carlson Specialties'

begin
	with	xmlnamespaces
		(
		N'http://milambda.net/legalPerson' as lp
		)
	select	*
		from	Person.PersonXml
		where	(Person.exist(N'/lp:legalPerson[1][@lp:abbreviatedName = sql:variable("@AbbreviatedName")]') = 1)
	;
end

set	statistics io	off
;
go



/* Clean up
drop index	ux_Person_PersonXml_CompanyNumber
	on	Person.PersonXml
;
go
alter table	Person.PersonXml
	drop column	CompanyNumber
;
go
drop index	ux_Person_PersonXml_SSN
	on	Person.PersonXml
;
go
alter table	Person.PersonXml
	drop column	SSN
;
go
drop function	Person.Attribute_Get_byLocalName
;
go
drop index	sxx_Person_PersonXml_Person
	on	Person.PersonXml
;
go
*/
